#!/bin/bash
#
#   USAGE:  build_fftw.sh <product_directory> <debug|opt|prof> [tar]
#   NOTE:  we build fftw with the native compiler

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
extraqual=${2}
maketar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=fftw
origpkgver=v3_3_6-pl2
pkgver=`echo ${origpkgver} | sed -e 's/-/_/g'`
ssibuildshims_version=v0_17_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics NO_BASE_QUAL

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
    exit 0
fi

echo
echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"
echo

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "debug" ]
then
    ccflg="-g -O0"
    extra_command=""
elif [ "${extraqual}" = "opt" ]
then
    ccflg="-O3 -g -DNDEBUG"
    extra_command="--enable-debug"
elif [ "${extraqual}" = "prof" ]
then
    ccflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
    extra_command=""
else
    echo "ERROR: please specify debug, opt, or prof"
    usage
    exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}
tar xf ${tardir}/${pkgtarname}.tar.gz

cd ${blddir}
# long double
rm -rf * 
CFLAGS="${ccflg}" ${sourcedir}/${pkgtarname}/configure --prefix=${pkgdir} \
                                 ${extra_command}  --enable-shared --enable-long-double
make -j ${ncore} || ssi_die "ERROR: make failed."
make check || ssi_die "ERROR: make check failed."
make install || ssi_die "ERROR: make install failed."
# quad precision
# note that quad precision requires gcc 4.6 or later
# skip if on SLF6 or OSX
if [[ ${OS1} == "Darwin" ]] || [[ ${OS} == "slf6-x86_64" ]] || [[ ${OS} == "rh7-ppc64le" ]];then
  echo
  echo "INFO: quad precision libraries not supported on ${OS}"
  echo
else
  rm -rf * 
  CFLAGS="${ccflg}" ${sourcedir}/${pkgtarname}/configure --prefix=${pkgdir} \
                                 ${extra_command}  --enable-shared --enable-quad-precision
  make -j ${ncore} || ssi_die "ERROR: make failed."
  make check || ssi_die "ERROR: make check failed."
  make install || ssi_die "ERROR: make install failed."
fi
# single precision build
rm -rf * 
CFLAGS="${ccflg}" ${sourcedir}/${pkgtarname}/configure --prefix=${pkgdir} \
                                 ${extra_command}  --enable-shared --enable-single
make -j ${ncore} || ssi_die "ERROR: make failed."
make check || ssi_die "ERROR: make check failed."
make install || ssi_die "ERROR: make install failed."
# normal build
rm -rf * 
CFLAGS="${ccflg}" ${sourcedir}/${pkgtarname}/configure --prefix=${pkgdir} \
                                 ${extra_command}  --enable-shared
make -j ${ncore} || ssi_die "ERROR: make failed."
make check || ssi_die "ERROR: make check failed."
make install || ssi_die "ERROR: make install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# Remove .la files.
find ${pkgdir} -name '*.la' | xargs rm -vf

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "FFTW_LIBRARY is ${FFTW_LIBRARY}"

# fix pkgconfig
for pkfilenm in fftw3.pc fftw3f.pc fftw3l.pc fftw3q.pc
do
  pkfile=${FFTW_LIBRARY}/pkgconfig/${pkfilenm}
  mv ${pkfile} ${pkfile}.bak
  cat ${pkfile}.bak | sed -e s"%${FFTW_FQ_DIR}%\${pcfiledir}/../..%" > ${pkfile}
done

# this must be last because it will exit
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
